package com.sxtvs.open.api.review.controller;

import com.sxtvs.open.api.chat.entity.KimiChat;
import com.sxtvs.open.api.chat.service.impl.KimiChatServiceImpl;
import com.sxtvs.open.api.review.service.impl.SSEService;
import com.sxtvs.open.core.advice.NoAPIResponse;
import com.sxtvs.open.core.auth.LoginRequired;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.io.IOException;
import java.util.concurrent.ThreadPoolExecutor;

@Controller
@RequestMapping("/user")
@NoAPIResponse
@Slf4j
public class SseController {

    @Resource
    private SSEService sseService;

    @Autowired
    private AliyunLogger logger;

    @Autowired
    private ThreadPoolExecutor poolExecutor;

    @GetMapping(path = "/sse", produces = MediaType.TEXT_EVENT_STREAM_VALUE, headers = "Cache-Control=no-cache")
    @NoAPIResponse
    public SseEmitter handleSse(@RequestParam("token") String token) throws IOException {
        SseEmitter sseEmitter = new SseEmitter(300 * 1000L); // 设置超时时间
        sseService.monitor(token, sseEmitter);
        sseEmitter.send(SseEmitter.event().name("data").data("data"));
        sseEmitter.onCompletion(() -> {
            log.info("连接关闭");
            sseEmitter.complete();
        });
        return sseEmitter;
    }

    @Resource
    private KimiChatServiceImpl kimiChatService;

    @PostMapping(path = "/chat", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    @NoAPIResponse
    public SseEmitter handleSsePost(@RequestBody KimiChat kimiChat) throws IOException {
        SseEmitter sseEmitter = new SseEmitter(0L); // 设置超时时间
        sseService.monitor(kimiChat.getGroupId(), sseEmitter);
        kimiChatService.create(kimiChat);
        sseEmitter.send(SseEmitter.event().name("hello").data("hello"));
        sseEmitter.complete();
        return sseEmitter;
    }

    @PostMapping(path = "/test", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    @NoAPIResponse
    public SseEmitter test(@RequestBody KimiChat kimiChat) throws IOException {
        SseEmitter sseEmitter = new SseEmitter(0L); // 设置超时时间
        sseService.monitor(kimiChat.getGroupId(), sseEmitter);

        sseEmitter.send(SseEmitter.event().name("hello").data("hello"));
        new Thread(() -> {
            try {
                sseEmitter.send(SseEmitter.event().name("hello").data("你好"));
                sseEmitter.send(SseEmitter.event().name("data").data("你好ya"));
            } catch (IOException e) {
                e.printStackTrace();
            }

        });
        return sseEmitter;
    }
}
